//
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "U1AutoUploadOperation.h"

#import <Crashlytics/Crashlytics.h>

#import "U1Asset.h"
#import "U1DataRepository.h"
#import "U1FileNode.h"
#import "U1FilesClient.h"
#import "U1FilesService.h"


@interface U1AutoUploadOperation ()
@property (getter=isExecuting) BOOL executing;
@property (getter=isFinished) BOOL finished;
- (void)finishExecuting;
- (void)beginUpload;
@end


@implementation U1AutoUploadOperation

@synthesize executing, finished, assetURL, assetID, folderID, filename, priority;

- (BOOL)isConcurrent;
{
	return YES;
}

- (void)finishExecuting;
{
	[self willChangeValueForKey:@"isExecuting"];
	[self willChangeValueForKey:@"isFinished"];
	self.finished = YES;
	self.executing = NO;
	[self didChangeValueForKey:@"isFinished"];
	[self didChangeValueForKey:@"isExecuting"];
}

- (void)start;
{
	[self willChangeValueForKey:@"isExecuting"];
	self.executing = YES;
	[self didChangeValueForKey:@"isExecuting"];
	
	if ([self isCancelled])
	{
		[self finishExecuting];
	}
	else
	{
		[self beginUpload];
	}
}

- (void)dealloc
{
	[assetURL release];
	[filename release];
	[assetID release];
	[folderID release];
    [super dealloc];
}

- (void)beginUpload;
{
	U1DataRepository *dataRepository = [U1DataRepository sharedDataRepository];
	[dataRepository dispatchBlockWithMainContext:^(NSManagedObjectContext *context) {
		
		U1Asset *asset = (id)[context objectWithID:self.assetID];
		U1FolderNode *folder = (id)[context objectWithID:self.folderID];
		
		[[U1FilesClient sharedFilesClient] uploadAssetURL:self.assetURL toFolder:folder withResourceName:self.filename withPriority:self.priority progressBlock:^(long long bytesUploaded, long long totalBytes) {
			
		} completionBlockOnMainQueue:^(U1FileNode *fileNode, NSError *error) {
			if (error != nil)
			{
				NSLog(@"U1AutoUploadOperation: auto-upload failed: %@", error);
			}
			else
			{
				NSLog(@"U1AutoUploadOperation - in completion block for upload, success.");
				asset.generation = asset.fileNode.generation;
				NSError *err = NULL;
				BOOL success = [dataRepository save:&err];
				if(!success){
					CLS_LOG(@"U1AutoUploadOperation: failed to save to repository: %@", err);
				}
			}
			[self finishExecuting];
		}];
		
	}];
}

@end
