//
//  Copyright 2012 Canonical Ltd.
//
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "U1MagicUploadDataProvider.h"

#import <CommonCrypto/CommonDigest.h>
#import <AssetsLibrary/AssetsLibrary.h>


@implementation U1MagicUploadDataProvider
{
	ALAssetsLibrary *library;
}

@synthesize assetURL;

- (id)init;
{
	self = [super init];
	if (self == nil)
		return nil;
	library = [[ALAssetsLibrary alloc] init];
	return self;
}

- (void)dealloc;
{
	[assetURL release];
	[library release];
	[super dealloc];
}

- (BOOL)isMagic;
{
	return YES;
}

- (void)prepareDataStreamWithBlock:(void(^)(NSInputStream *dataStream, NSString *mimeType, NSUInteger length, NSError *error))block;
{
	[library assetForURL:self.assetURL resultBlock:^(ALAsset *asset) {
		
		if (asset == nil)
		{
			if (block)
			{
				NSError *error = [NSError errorWithDomain:@"nonexistentAsset" code:1 userInfo:nil];
				block(nil, nil, 0, error);
			}
		}
		else
		{
			// Note: Doesn't currently store the hash and magicHash on the U1Asset. Soon.

			ALAssetRepresentation *representation = [asset defaultRepresentation];
			uint8_t *buffer = (uint8_t*)malloc(representation.size);
			NSUInteger length = [representation getBytes:buffer fromOffset:0 length:representation.size error:nil];
			NSData *imageData = [NSData dataWithBytesNoCopy:buffer length:length freeWhenDone:YES];
				   			
			unsigned char hashDigest[CC_SHA1_DIGEST_LENGTH];
				   
			CC_SHA1([imageData bytes], length, hashDigest);
			
			NSMutableString *hashDigestString = [NSMutableString string];
			for (int i =0; i < CC_SHA1_DIGEST_LENGTH; i++)
			{
				[hashDigestString appendFormat:@"%02x", hashDigest[i]];
			}

			unsigned char magicDigest[CC_SHA1_DIGEST_LENGTH];
				   
			CC_SHA1_CTX magicCtx;
			CC_SHA1_Init(&magicCtx);
			CC_SHA1_Update(&magicCtx, "Ubuntu One", 10);
			CC_SHA1_Update(&magicCtx, [imageData bytes], length);
			CC_SHA1_Final(magicDigest, &magicCtx);

			NSMutableString *magicDigestString = [NSMutableString string];
			for (int i =0; i < CC_SHA1_DIGEST_LENGTH; i++)
			{
				[magicDigestString appendFormat:@"%02x", magicDigest[i]];
			}
            
			NSDictionary *magicDict = [NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"file", hashDigestString, magicDigestString, nil] forKeys:[NSArray arrayWithObjects:@"kind", @"hash", @"magic_hash", nil]];
			NSData *magicDictData = [NSJSONSerialization dataWithJSONObject:magicDict options:0 error:nil];
            NSString *magicDictAsJSON = [[NSString alloc] initWithData:magicDictData encoding:NSUTF8StringEncoding];
			NSInputStream *dataStream = [NSInputStream inputStreamWithData:[magicDictAsJSON dataUsingEncoding:NSUTF8StringEncoding]];
				   
			if (block)
				block(dataStream, @"application/json", [[magicDictAsJSON dataUsingEncoding:NSUTF8StringEncoding] length], nil);
		}
		
	} failureBlock:^(NSError *error) {
		if (block)
			block(nil, nil, 0, error);
    }];
}

@end
